/**
 * Renders all the input elements in a form.
 * @param {Object} form
 * @return {String}
 */
function _drupalgap_form_render_elements(form) {
  try {
    //dpm('_drupalgap_form_render_elements');
    //console.log(form);
    var content = '';
    var content_sorted = '';
    var content_weighted = [];
    // For each form element, if the element objects name property isn't set,
    // set it, then render the element if access is permitted. While rendering
    // the elements, set them aside according to their widget weight so they
    // can be appended to the content string in the correct order later.
    for (var name in form.elements) {
        if (!form.elements.hasOwnProperty(name)) { continue; }
        var element = form.elements[name];
        //dpm(name);
        //console.log(element);
        if (!element.name) { element.name = name; }
        if (drupalgap_form_element_access(element)) {
          if (
            element.is_field &&
            typeof element.field_info_instance.widget.weight !== 'undefined'
          ) {
            content_weighted[element.field_info_instance.widget.weight] =
              _drupalgap_form_render_element(form, element);
          }
          else {
            // Extract the bundle. Note, on comments the bundle is prefixed with
            // 'comment_node_' so we need to remove that to correctly map to the
            // potential extra fields data.
            var bundle = null;
            if (form.bundle) {
              bundle = form.bundle;
              if (
                form.entity_type == 'comment' &&
                form.bundle.indexOf('comment_node_') != -1
              ) { bundle = form.bundle.replace('comment_node_', ''); }
            }
            // This is not a field, if it has a weight in
            // field_info_extra_fields use it, otherwise just append it to the
            // content.
            if (
              form.entity_type && bundle &&
              typeof drupalgap.field_info_extra_fields[bundle][name] !==
                'undefined' &&
              typeof
                drupalgap.field_info_extra_fields[bundle][name].weight !==
                'undefined'
            ) {
              var weight =
                drupalgap.field_info_extra_fields[bundle][name].weight;
              content_weighted[weight] =
              _drupalgap_form_render_element(form, element);
            }
            else { content += _drupalgap_form_render_element(form, element); }
          }
        }
    }
    // Prepend the weighted elements to the content.
    if (!empty(content_weighted)) {
      for (var weight in content_weighted) {
        content_sorted += content_weighted[weight] + '\n';
      }
      // Attach sorted content.
      content = content_sorted + '\n' + content;
    }
    // Add any form buttons to the form elements html, if access to the button
    // is permitted.
    if (form.buttons && form.buttons.length != 0) {
      for (var name in form.buttons) {
          if (!form.buttons.hasOwnProperty(name)) { continue; }
          var button = form.buttons[name];
          if (drupalgap_form_element_access(button)) {
            var attributes = {
              type: 'button',
              id: drupalgap_form_get_element_id(name, form.id)
            };
            if (button.attributes) { $.extend(attributes, button.attributes); }
            content += '<button ' + drupalgap_attributes(attributes) + '">' +
              button.title +
            '</button>';
          }
      }
    }
    return content;
  }
  catch (error) { console.log('_drupalgap_form_render_elements - ' + error); }
}

/**
 * Renders an input element for a form.
 * @param {Object} form
 * @param {Object} element
 * @return {String}
 */
function _drupalgap_form_render_element(form, element) {
  try {
    //dpm('_drupalgap_form_render_element');
    //console.log(element);
    // @TODO don't wrap hidden inputs in divs.
    var attrs = drupalgap_attributes({
        'dg-form-element': '',
        element_name: element.name,
        'class': drupalgap_form_get_element_container_class(name)
    });
    return '<div ' + attrs + '></div>';
  }
  catch (error) { console.log('_drupalgap_form_render_element - ' + error); }
}

/**
 * @param {Object} element
 * @param {String|undefined} language
 * @param {Number|undefined} delta
 */
function dg_form_element_ng_model(element, language, delta) {
  try {
    var model_path = "form_state.values['" + element.name + "']";
    if (element.is_field) {
      model_path += '.' + language + '[' + delta + '].value';
    }
    return model_path;
  }
  catch (error) { console.log('dg_form_element_ng_model - ' + error); }
}

dgApp.directive("dgFormElement", function($compile, $injector) {
    //dpm('dgFormElement');
    return {
      controller: function($scope) { },
      link: function($scope, $element) {
        
        //dpm('dgFormElement - link...');
        //console.log(arguments);

        // Extract the form, then the element from the form.
        var form = $scope.form;        
        var element = form.elements[$element.attr('element_name')]; // @TODO just use "name" here
        
        //console.log(element.name);
        //console.log(arguments);
        //console.log(element);
        
        var html = '';
    
        if (!element) { return html; }

        // Extract the element name.
        var name = element.name;

        // Grab the language.
        var language = language_default();
        $scope.language = language;

        // We'll assume the element has no items (e.g. title, nid, vid, etc), unless
        // we determine later that this element is a field, then it'll have items.
        var items = false;

        // If this element is a field, extract the items from the language code and
        // determine what module and hook will handle the items. If the element is
        // not a field, just flatten it into a single item collection and determine
        // which module handles this element type. Keep in mind not all the modules
        // actually exist, and we've placed implementations into the field module.
        var module = false;
        var field_widget_form_function_name = false;
        var field_widget_form_function = false;
        if (element.is_field) {
          items = element[language];
          module = element.field_info_instance.widget.module;
        }
        else {
          items = {0: element};
          module = drupalgap_form_element_get_module_name(element.type);
        }
        
        // If we have a module, determine the widget form function name.
        if (module) {
          field_widget_form_function_name = module + '_field_widget_form';
        }

        // If there were no items, just return.
        if (!items || items.length == 0) { return html; }
    
        // Generate default variables. Set the angular model to the form state
        // values for the element's name.
        var variables = {
          attributes: {
            
          }
        };
    
        // Grab the info instance and info field for the field, then attach them
        // both to the variables object so all theme functions will have access
        // to that data.
        variables.field_info_field = element.field_info_field;
        variables.field_info_instance = element.field_info_instance;
    
        // Render the element item(s). Remember the final delta value for later.
        var delta = 0;
        var item_html = '';
        var item_label = '';
        var render_item = null;
        for (var delta in items) {
            if (!items.hasOwnProperty(delta)) { continue; }
            var item = items[delta];

            // Build the ng-model for the element, if it hasn't already been set.
            if (!variables.attributes['ng-model']) {
              var ng_model = dg_form_element_ng_model(element, language, delta);
              variables.attributes['ng-model'] = ng_model;
            }

            // We'll render the item, unless we prove otherwise.
            render_item = true;

            // Overwrite the variable's attributes id with the item's id.
            variables.attributes.id = item.id;

            // Attach the item as the element onto variables.
            variables.element = item;

            // Create an array for the item's children if it doesn't exist already.
            // This is used by field widget forms to extend form elements.
            if (!items[delta].children) { items[delta].children = []; }

            // Generate the label for field items on delta zero only. Keep in mind
            // rendered labels, with an element title_placeholder set to true,
            // will not be appended to the result html later.
            if (element.is_field && delta == 0) {
              item.title = element.title;
              item_label = theme('form_element_label', {'element': item});
            }

            // If the element's title is set to be a placeholder, set the
            // placeholder attribute equal to the title on the current item, unless
            // someone already set it. If it is a required element, mark it as such.
            if (
              delta == 0 && typeof element.title_placeholder !== 'undefined' &&
              element.title_placeholder &&
              typeof variables.attributes['placeholder'] === 'undefined'
            ) {
              var placeholder = element.title;
              // @TODO show a better required marker for placeholders.
              /*if (element.required) {
                placeholder += ' ' + theme('form_required_marker', { });
              }*/
              variables.attributes['placeholder'] = placeholder;
            }

            // If there wasn't a default value provided, set one. Then set the
            // default value into the variables' attributes. Although, if we have an
            // item value, just use that.
            if (!item.default_value) { item.default_value = ''; }
            variables.attributes.value = item.default_value;
            if (typeof item.value !== 'undefined') {
              variables.attributes.value = item.value;
            }

            // If this is a field, is there a directive available to handle the
            // field's widget form?
            if (element.is_field) {
              var directive_name = dg_get_camel_case(field_widget_form_function_name);
              if (dg_directive_exists($injector, directive_name)) {

                // Place variables into the scope so the hookFieldWidgetForm
                // implementor will have access to them.
                //$scope.form = form;
                //$scope.form_state = null;
                $scope.field = element.field_info_field;
                $scope.instance = element.field_info_instance;
                $scope.items = items;
                $scope.delta = delta;
                //$scope.element = element;

                // Set up basic widget attributes.
                var attrs = {
                  field_name: name,
                  field_widget_form: field_widget_form_function_name,
                  delta: delta,
                  language: language
                };

                // Add the directive's attribute and the div container.
                attrs[field_widget_form_function_name.replace(/_/g, '-')] = '';
                item_html += '<div ' + drupalgap_attributes(attrs) + '>{{' + name + '}}</div>';

              }
              else {
                console.log('WARNING: you must change ' + field_widget_form_function_name + '() to a directive called ' + directive_name);
              }
            }
    
            // HOOK_FIELD_WIDGET_FORM VIEW
            // Call the hook_field_widget_form() if necessary. Merge any changes
            // to the item back into this item.
            if (field_widget_form_function) {

              /*
              // @TODO - sometimes an item gets merged without a type here, why?
              // @UPDATE - did the recursive extend fix this?
              item = $.extend(true, item, items[delta]);
              // If the item type got lost, replace it.
              if (!item.type && element.type) { item.type = element.type; }*/
            }

            // Merge element attributes into the variables object.
            /*if (item.options && item.options.attributes) {
              variables.attributes = angular.extend(
                {},
                variables.attributes,
                item.options.attributes
              );
            }*/

            // Render only "non field" element items here, any field items will be
            // taken care of by the hook_field_widget_form controller..
            if (!element.is_field) {

              item_html = _drupalgap_form_render_element_item(
                form,
                element,
                variables,
                item
              );

            }
            if (typeof item_html === 'undefined') {
              render_item = false;
              break;
            }
        }

        // Set the variables onto the scope.
        $scope.variables = variables;

        // Are we skipping the render of the item?
        if (!render_item) { return ''; }
    
        // Is this element wrapped? We won't wrap hidden inputs by default, unless
        // someone is overriding it.
        var wrapped = true;
        if (typeof element.wrapped !== 'undefined' && !element.wrapped) {
          wrapped = false;
        }
        if (element.type == 'hidden') {
          wrapped = false;
          if (element.wrapped) { wrapped = true; }
        }
    
        // If there is an element prefix, place it in the html.
        if (element.prefix) { html += element.prefix; }
    
        // Open the element container.
        /*var container_attributes = {
          'class': drupalgap_form_get_element_container_class(name)
        };
        if (wrapped) {
          html += '<div ' + drupalgap_attributes(container_attributes) + '>';
        }*/
    
        // Add a label to the element, except submit and hidden elements. Any field
        // labels have already been rendered, other element labels must be manually
        // rendered here. Don't attach the label if the element's title_placeholder
        // is set to true.
        if (element.type != 'submit' && element.type != 'hidden') {
          if (
            typeof element.title_placeholder !== 'undefined' &&
            element.title_placeholder
          ) { /* Skip label for placeholders. */ }
          else {
            if (element.is_field) { html += item_label; }
            else {
              html += theme('form_element_label', {'element': element});
            }
          }
        }
    
        // Add the item html if it isn't empty.
        if (item_html != '') { html += item_html; }
    
        // Add element description.
        if (element.description && element.type != 'hidden') {
          html += '<div>' + t(element.description) + '</div>';
        }
    
        // Close the element container.
        //if (wrapped) { html += '</div>'; }
    
        // If there is an element suffix, place it in the html.
        if (element.suffix) { html += element.suffix; }
        
        // Compile the template for Angular and append it to the directive's
        // html element.
        var linkFn = $compile(html);
        var content = linkFn($scope);
        $element.append(content);

      }
    };
});

/**
 * Given a form, an element, the variables for a theme function, and the element
 * item, this will return the html rendering of the element item.
 * @param {Object} form
 * @param {Object} element
 * @param {Object} variables
 * @param {Object} item
 * @return {*}
 */
function _drupalgap_form_render_element_item(form, element, variables, item) {
  try {
    //dpm('_drupalgap_form_render_element_item');
    //dpm(element.name);
    //console.log(arguments);
    
    var html = '';
    // Depending on the element type, if necessary, adjust the variables and/or
    // theme function to be used, then render the element by calling its theme
    // function.
    // @TODO - this block of code should be moved into their respective
    // implementations of hook_field_widget_form().
    switch (item.type) {
      case 'text':
        item.type = 'textfield';
        break;
      case 'list_text':
      case 'list_float':
      case 'list_integer':
        item.type = 'select';
        break;
    }

    // Set the theme function.
    var theme_function = item.type;

    // If the element is disabled, add the 'disabled' attribute.
    if (element.disabled) { variables.attributes.disabled = ''; }

    // Merge the item into variables.
    $.extend(true, variables, item);

    // If a value isn't set on variables, try to set it with the default value
    // on the item.
    if (typeof variables.value === 'undefined' || variables.value == null) {
      if (typeof item.default_value !== 'undefined') {
        variables.value = item.default_value;
      }
    }

    // Run the item through the theme system if a theme function exists, or try
    // to use the item markup, or let the user know the field isn't supported.
    if (drupalgap_function_exists('theme_' + theme_function)) {
      html += theme(theme_function, variables);
    }
    else {
      if (item.markup || item.markup == '') { html += item.markup; }
      else {
        // @todo - the reason for this warning sometimes happens because the
        // item.type is lost with $.extend in _drupalgap_form_render_element().
        // @update - if an item doesn't have a type, it gets set by the parent
        // element, so we should now always have a type available here.
        var msg = 'Field ' + item.type + ' not supported.';
        console.log('WARNING: _drupalgap_form_render_element_item() - ' + msg);
        dpm(item);
        return null;
      }
    }

    // Render any item children. If the child has markup, just use the html,
    // otherwise run the child through theme().
    if (item.children && item.children.length > 0) {
      for (var i = 0; i < item.children.length; i++) {
        if (item.children[i].markup) { html += item.children[i].markup; }
        else if (item.children[i].type || item.children[i].theme) {
          var theme_type = item.children[i].type;
          if (!theme_type) { theme_type = item.children[i].theme; }
          // Is there a title for a label?
          if (item.children[i].title) {
            html += theme('form_element_label', {
                element: item.children[i]
            });
          }
          // Render the child with the theme system.
          if (item.children[i].prefix) { html += item.children[i].prefix; }
          html += theme(theme_type, item.children[i]);
          if (item.children[i].suffix) { html += item.children[i].suffix; }
        }
        else {
          console.log(
            'WARNING: _drupalgap_form_render_element_item - ' +
            'failed to render child ' + i + ' for ' + element.name
          );
        }
      }
    }

    return html;
  }
  catch (error) {
    console.log('_drupalgap_form_render_element_item - ' + error);
  }
}

/*dgControllers.controller('drupalgapFormElementController', ['$scope', '$element', '$sce',
  function($scope, $element, $sce) {

    dpm('drupalgapFormElementController');
    console.log(arguments);
    
    
    
  }]);*/

/**
 * Given a form element, this will return true if access to the element is
 * permitted, false otherwise.
 * @param {Object} element
 * @return {Boolean}
 */
function drupalgap_form_element_access(element) {
  try {
    var access = true;
    if (element.access == false) { access = false; }
    return access;
  }
  catch (error) { console.log('drupalgap_form_element_access - ' + error); }
}

/**
 * Given a form element type, this will return the name of the module that
 * implements the hook_field_widget_form() for the element. Keep in mind for now
 * some of the module names don't exist, and are actually implemented inside
 * the field module. If no module is found, it returns false.
 * @param {String} type
 * @return {String}
 */
function drupalgap_form_element_get_module_name(type) {
  try {
    var module = false;
    switch (type) {
      case 'checkbox':
      case 'radios':
      case 'select':
        module = 'options';
        break;
      case 'image':
        module = 'image';
        break;
    }
    return module;
  }
  catch (error) {
    console.log('drupalgap_form_element_get_module_name - ' + error);
  }
}

/**
 * Given an element name, this will return the class name to use on the
 * element's container.
 * @param {String} name
 * @return {String}
 */
function drupalgap_form_get_element_container_class(name) {
  try {
    return 'form-item field-name-' + name.replace(/_/g, '-');
  }
  catch (error) {
    console.log('drupalgap_form_get_element_container_class - ' + error);
  }
}

/**
 * Given an element name, the form, a language code and a delta value, this
 * will return default values that can be used to place an item element into a
 * Forms API object.
 * @param {String} name
 * @param {Object} form
 * @param {String} language
 * @param {Number} delta
 * @return {Object}
 */
function drupalgap_form_element_item_create(name, form, language, delta) {
  try {
    // Generate the id for this element field item and set it and
    // some default options onto the element item.
    var id = drupalgap_form_get_element_id(name, form.id, language, delta);
    return {
      id: id,
      options: {
        attributes: {
          id: id
        }
      },
      required: form.elements[name].required
    };
  }
  catch (error) {
    console.log('drupalgap_form_element_item_create - ' + error);
  }
}

/**
 *
 * @param {Object} form
 * @param {Object} form_state
 * @param {Object} element
 * @param {String} language
 * @param {Number} delta
 * @return {Array}
 */
function _drupalgap_form_element_items_widget_arguments(form, form_state,
  element, language, delta) {
  try {
    var widget_arguments = [];
    widget_arguments.push(form); // form
    widget_arguments.push(form_state); // form state
    widget_arguments.push(element.field_info_field); // field
    widget_arguments.push(element.field_info_instance); // instance
    widget_arguments.push(language); // language
    widget_arguments.push(form.elements[element.name][language]); // items
    widget_arguments.push(delta); // delta
    widget_arguments.push(element); // element
    return widget_arguments;
  }
  catch (error) {
    console.log('_drupalgap_form_element_items_widget_arguments - ' + error);
  }
}

